import 'dart:convert';

import 'package:flutter/services.dart';

class DataTrackingConfig {
  DataTrackingConfig(
      {bool collectLocation = _DataConfigDefaults.collectLocation,
      bool collectAaid = _DataConfigDefaults.collectAaid,
      bool collectIdfa = _DataConfigDefaults.collectIdfa}) {
    _collectLocation = collectLocation;
    _collectAaid = collectAaid;
    _collectIdfa = collectIdfa;
  }

  bool _collectLocation = _DataConfigDefaults.collectLocation;
  bool _collectAaid = _DataConfigDefaults.collectAaid;
  bool _collectIdfa = _DataConfigDefaults.collectIdfa;
  DataTracker? _tracker;

  bool get collectLocation {
    return _collectLocation;
  }

  set collectLocation(bool value) {
    _collectLocation = value;
    _tracker?.setConfig(this);
  }

  bool get collectAaid {
    return _collectAaid;
  }

  set collectAaid(bool value) {
    _collectAaid = value;
    _tracker?.setConfig(this);
  }

  bool get collectIdfa {
    return _collectIdfa;
  }

  set collectIdfa(bool value) {
    _collectIdfa = value;
    _tracker?.setConfig(this);
  }
}

class DataTracker {
  DataTracker(this._channel);
  final MethodChannel _channel;

  static const String _configStringKey = 'config';
  static const String _defaultTrackerName = 'default';
  static const String _collectLocationKey = 'collectLocation';
  static const String _collectAaidKey = 'collectAaid';
  static const String _collectIdfaKey = 'collectIdfa';

  Future<DataTrackingConfig> getConfig() async {
    final configString = await _getConfigStringFromNative();
    if (configString == null) {
      final _config = DataTrackingConfig().._tracker = this;
      return _config;
    }
    final config = jsonDecode(configString) as Map<String, dynamic>;
    final defaultTracker = config[_defaultTrackerName] as Map<String, dynamic>;
    final collectLocation = defaultTracker[_collectLocationKey] as bool? ??
        _DataConfigDefaults.collectLocation;
    final collectAaid = defaultTracker[_collectAaidKey] as bool? ??
        _DataConfigDefaults.collectAaid;
    final collectIdfa = defaultTracker[_collectIdfaKey] as bool? ??
        _DataConfigDefaults.collectIdfa;

    return DataTrackingConfig(
        collectLocation: collectLocation,
        collectAaid: collectAaid,
        collectIdfa: collectIdfa)
      .._tracker = this;
  }

  Future<void> setConfig(DataTrackingConfig config) async {
    final defaultTrackerConfig = <String, bool>{
      _collectLocationKey: config.collectLocation,
      _collectAaidKey: config.collectAaid,
      _collectIdfaKey: config.collectIdfa,
    };
    final configMap = {_defaultTrackerName: defaultTrackerConfig};
    await _setConfigStringToNative(jsonEncode(configMap));
  }

  Future<String?> _getConfigStringFromNative() async {
    try {
      return await _channel
          .invokeMethod<String?>('getDataTrackingConfig', <String, dynamic>{});
    } on PlatformException catch (error) {
      print(error.message);
      return null;
    }
  }

  Future<void> _setConfigStringToNative(String configString) async {
    try {
      await _channel.invokeMethod<String?>(
          'setDataTrackingConfig', {_configStringKey: configString});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }
}

class _DataConfigDefaults {
  static const collectLocation = false;
  static const collectAaid = true;
  static const collectIdfa = true;
}
